(*
 * Copyright 2018 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *)

[inherit('mysql', 'common', 'pmysql')]
module pmysql2;

type
   vararg = record
               typ : integer;
               val : integer_address;
               len : integer;
               vallen : integer_address;
            end;

function long_arg(var v : integer) : vararg;

var
   res : vararg;

begin
   res.typ := MYSQL_TYPE_LONG;
   res.val := iaddress(v);
   long_arg := res;
end;

function string_arg(var v : pstr) : vararg;

var
   res : vararg;

begin
   res.typ := MYSQL_TYPE_STRING;
   res.val := iaddress(v.body);
   res.len := v.length;
   string_arg := res;
end;

function longstring_arg(var v : longpstr) : vararg;

var
   res : vararg;

begin
   res.typ := MYSQL_TYPE_STRING;
   res.val := iaddress(v.body);
   res.len := upper(v.body);
   res.vallen := iaddress(v.length);
   longstring_arg := res;
end;

procedure pmysql_init_bind_in(var bp : mysql_bind; a : vararg);

begin
   if a.typ = MYSQL_TYPE_STRING then begin
      pmysql_init_bind_stringadr_in(bp, a.val, a.len);
   end else begin
      pmysql_init_bind_nonstring(bp, a.typ, a.val);
   end;
end;

procedure pmysql_init_bind_out(var bp : mysql_bind; a : vararg);

begin
   if a.typ = MYSQL_TYPE_STRING then begin
      pmysql_init_bind_stringadr_out(bp, a.val, a.len, a.vallen);
   end else begin
      pmysql_init_bind_nonstring(bp, a.typ, a.val);
   end;
end;

function pmysql_prepare(con : mysql_ptr; sql : pstr; args : [LIST] vararg) : mysql_stmt_ptr;

var
   stmt : mysql_stmt_ptr;
   i : integer;
   va : vararg;
   inparam : array [1..3200] of mysql_bind;
   stat : integer;

begin
   stmt := pmysql_stmt_init(con);
   if stmt = 0 then return 0;
   stat := pmysql_stmt_prepare(stmt, sql);
   if stat <> 0 then return 0;
   for i := 1 to argument_list_length(args) do begin
      va := argument(args, i);
      pmysql_init_bind_in(inparam[i], va);
   end;
   stat := pmysql_stmt_bind_param(stmt, inparam);
   if stat <> 0 then return 0;
   pmysql_prepare := stmt;
end;

function pmysql_get_result_single_long(stmt : mysql_stmt_ptr; var res : integer) : boolean;

var
   stat : integer;
   outparam : array[1..1] of mysql_bind;

begin
   stat := pmysql_stmt_execute(stmt);
   if stat <> 0 then return false;
   pmysql_init_bind_long(outparam[1], res);
   stat := pmysql_stmt_bind_result(stmt, outparam);
   if stat <> 0 then return false;
   stat := pmysql_stmt_store_result(stmt);
   if stat <> 0 then return false;
   pmysql_get_result_single_long := (pmysql_stmt_fetch(stmt) = 0);
end;

function pmysql_get_result_single_string(stmt : mysql_stmt_ptr; var res : longpstr) : boolean;

var
   stat : integer;
   outparam : array[1..1] of mysql_bind;

begin
   stat := pmysql_stmt_execute(stmt);
   if stat <> 0 then return false;
   pmysql_init_bind_string_out(outparam[1], res);
   stat := pmysql_stmt_bind_result(stmt, outparam);
   if stat <> 0 then return false;
   stat := pmysql_stmt_store_result(stmt);
   if stat <> 0 then return false;
   pmysql_get_result_single_string := (pmysql_stmt_fetch(stmt) = 0);
end;

function pmysql_get_result_start(stmt : mysql_stmt_ptr; args : [LIST] vararg) : integer;

var
   i : integer;
   va : vararg;
   stat : integer;
   outparam : array[1..100] of mysql_bind;

begin
   stat := pmysql_stmt_execute(stmt);
   if stat <> 0 then return stat;
   
   for i := 1 to argument_list_length(args) do begin
      va := argument(args, i);
      pmysql_init_bind_out(outparam[i], va);
   end;
   
   stat := pmysql_stmt_bind_result(stmt, outparam);
   if stat <> 0 then return stat;
   stat := pmysql_stmt_store_result(stmt);
   if stat <> 0 then return stat;
   pmysql_get_result_start := 0;
end;

end.
