[inherit('pmysql2', 'pmysql', 'common', 'mysql')]
program test2(input, output);

procedure checkcon(con : mysql_ptr);

begin
   if con = 0 then begin
      writeln(pmysql_error(con));
      halt;
   end;
end;

procedure checkstmt(stmt : mysql_stmt_ptr; con : mysql_ptr);

begin
   if stmt = 0 then begin
      writeln(pmysql_error(con));
      halt;
   end;
end;

procedure checkstat(stat : integer; stmt : mysql_stmt_ptr);

begin
   if stat <> 0 then begin
      writeln(pmysql_stmt_error(stmt));
      halt;
   end;
end;

function connect(host, un, pw, db : pstr) : mysql_ptr;

var
   con : mysql_ptr;

begin
   con := pmysql_init;
   checkcon(con);
   con := pmysql_real_connect(con, host, un, pw, db);
   checkcon(con);
   connect := con;
end;

procedure test_getone(con : mysql_ptr; f1 : integer);

var
   stmt : mysql_stmt_ptr;
   f2 : longpstr(255);

begin
   stmt := pmysql_prepare(con, 'SELECT f2 FROM t1 WHERE f1 = ?', long_arg(f1));
   checkstmt(stmt, con);
   if pmysql_get_result_single_string(stmt, f2) then begin
      writeln(stdstr(f2));
   end;
   pmysql_stmt_free_result(stmt);
end;

procedure test_getall(con : mysql_ptr);           

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   f1 : integer;
   f2 : longpstr(255);

begin
   stmt := pmysql_prepare(con, 'SELECT f1,f2 FROM t1');
   checkstmt(stmt, con);
   stat := pmysql_get_result_start(stmt, long_arg(f1), longstring_arg(f2));
   checkstat(stat, stmt);
   while pmysql_stmt_fetch(stmt) = 0 do begin
      writeln(f1, ' ', stdstr(f2));
   end;
   pmysql_stmt_free_result(stmt);
end;

procedure test_insert(con : mysql_ptr; f1 : integer; f2 : pstr);

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   inparam : array[1..2] of mysql_bind;

begin
   stmt := pmysql_prepare(con, 'INSERT INTO t1 VALUES(?, ?)', long_arg(f1), string_arg(f2));
   checkstmt(stmt, con);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   writeln(pmysql_stmt_affected_rows(stmt), ' rows inserted');
   pmysql_stmt_free_result(stmt);
end;

procedure test_delete(con : mysql_ptr; f1 : integer);

var
   stmt : mysql_stmt_ptr;
   stat : integer;
   inparam : array[1..1] of mysql_bind;

begin
   stmt := pmysql_prepare(con, 'DELETE FROM t1 WHERE f1 = ?', long_arg(f1));
   checkstmt(stmt, con);
   stat := pmysql_stmt_execute(stmt);
   checkstat(stat, stmt);
   writeln(pmysql_stmt_affected_rows(stmt), ' rows deleted');
   pmysql_stmt_free_result(stmt);
end;

var
   con : mysql_ptr;

begin
   con := connect('localhost', 'root', 'hemmeligt', 'test');
   test_getone(con, 2);
   test_getall(con);
   test_insert(con, 999, 'XXX');
   test_getall(con);
   test_delete(con, 999);
   test_getall(con);
   pmysql_close(con);
end.
